@@ -713,28 +713,18 @@ void handle_quota_options(ext2_filsys fs)
 
 	if (usrquota == QOPT_ENABLE && !fs->super->s_usr_quota_inum) {
 		if ((qf_ino = quota_file_exists(fs, USRQUOTA,
-						QFMT_VFS_V1)) > 0) {
-			if (quota_update_inode(qctx, qf_ino, USRQUOTA) == 0)
-				quota_set_sb_inum(fs, qf_ino, USRQUOTA);
-			else
-				quota_write_inode(qctx, USRQUOTA);
-		} else {
-			quota_write_inode(qctx, USRQUOTA);
-		}
+						QFMT_VFS_V1)) > 0)
+			quota_update_limits(qctx, qf_ino, USRQUOTA);
+		quota_write_inode(qctx, USRQUOTA);
 	} else if (usrquota == QOPT_DISABLE) {
 		quota_remove_inode(fs, USRQUOTA);
 	}
 
 	if (grpquota == QOPT_ENABLE && !fs->super->s_grp_quota_inum) {
 		if ((qf_ino = quota_file_exists(fs, GRPQUOTA,
-						QFMT_VFS_V1)) > 0) {
-			if (quota_update_inode(qctx, qf_ino, GRPQUOTA) == 0)
-				quota_set_sb_inum(fs, qf_ino, GRPQUOTA);
-			else
-				quota_write_inode(qctx, GRPQUOTA);
-		} else {
-			quota_write_inode(qctx, GRPQUOTA);
-		}
+						QFMT_VFS_V1)) > 0)
+			quota_update_limits(qctx, qf_ino, GRPQUOTA);
+		quota_write_inode(qctx, GRPQUOTA);
 	} else if (grpquota == QOPT_DISABLE) {
 		quota_remove_inode(fs, GRPQUOTA);
 	}
@@ -744,7 +734,8 @@ void handle_quota_options(ext2_filsys fs)
 	if ((usrquota == QOPT_ENABLE) || (grpquota == QOPT_ENABLE)) {
 		fs->super->s_feature_ro_compat |= EXT4_FEATURE_RO_COMPAT_QUOTA;
 		ext2fs_mark_super_dirty(fs);
-	} else if ((usrquota == QOPT_DISABLE) && (grpquota == QOPT_DISABLE)) {
+	} else if (!fs->super->s_usr_quota_inum &&
+		   !fs->super->s_grp_quota_inum) {
 		fs->super->s_feature_ro_compat &= ~EXT4_FEATURE_RO_COMPAT_QUOTA;
 		ext2fs_mark_super_dirty(fs);
 	}
@@ -754,7 +745,7 @@ void handle_quota_options(ext2_filsys fs)
 
 void parse_quota_opts(const char *opts)
 {
-	char	*buf, *token, *next, *p, *arg;
+	char	*buf, *token, *next, *p;
 	int	len;
 
 	len = strlen(opts);
